#!/usr/bin/env python
#-*- coding:utf-8 -*-

import os

class Inifile():
    """
    ini file to read and write.
    """
    def __init__(self, filename):
        """ @param filename: the full path name. """
        self.ini = []
        self.filename = filename
        if os.path.exists(filename):
            f = open(filename)
            x = f.read()
            f.close()
            lines = x.split("\n")
            section = ""
            for line in lines:
                if line == "":
                    continue
                if len(line) > 2 and line[0] =="[" and line[-1] == "]":
                    section = line[1:-1]
                elif section == "":
                    pass # 何もしない
                elif "=" in line:
                    pos = line.index("=")
                    self.__add(section, line[:pos], line[pos+1:])
    
    def __add(self, section, key, value):
        for dic1 in self.ini:
            if section in dic1.keys():
                for dic2 in dic1[section]:
                    if key in dic2.keys():
                        dic2[key] = value
                        return
                dic1[section].append({key: value})
                return
        self.ini.append({section: [{key: value}]})
    
    def __get_value(self, section, key):
        for dic1 in self.ini:
            if section in dic1.keys():
                for dic2 in dic1[section]:
                    if key in dic2.keys():
                        return dic2[key]
        return None
    
    def save(self):
        """ Save the contents. """
        s = ""
        for dic1 in self.ini:
            for section in dic1.keys():
                s += "[" + section + "]" + "\n"
                for dic2 in dic1[section]:
                    for key in dic2.keys():
                        s += key + "=" + dic2[key] + "\n"
                s += "\n"
        if s != "":
            f = open(self.filename, "w")
            f.write(s)
            f.close()
    
    def section_exists(self, section):
        """ Check existence of section. """
        for dic1 in self.ini:
            if section in dic1.keys():
                return True
        return False
    
    def erase_section(self, section):
        """ Remove the specified section. """
        for dic1 in self.ini:
            if section in dic1.keys():
                dic1.pop(section)
                return True
        return False
    
    def read_int(self, section, key, default):
        """ @param default: int. """
        result = self.__get_value(section, key)
        if result == None:
            return default
        try:
            return int(result)
        except:
            raise ValueError, '設定ファイルの "%s", "%s" の値が int ではありません' % (section, key)
    
    def read_float(self, section, key, default):
        """ @param default: float. """
        result = self.__get_value(section, key)
        if result == None:
            return default
        try:
            return float(result)
        except:
            raise ValueError, '設定ファイルの "%s", "%s" の値が float ではありません' % (section, key)
    
    def read_bool(self, section, key, default):
        """ @param default: bool. """
        result = self.__get_value(section, key)
        if result == None:
            return default
        if result == "1":
            return True
        elif result =="0":
            return False
        else:
            raise ValueError, '設定ファイルの "%s", "%s" の値が bool ではありません' % (section, key)
    
    def read_str(self, section, key, default):
        """ @param default: str. """
        result = self.__get_value(section, key)
        if result == None:
            return default
        return result
    
    def write_int(self, section, key, value):
        """ @param value: int. """
        if isinstance(value, int):
            self.__add(section, key, str(value))
        else:
            raise ValueError, '"%s", "%s" の値が int ではありません' % (section, key)
    
    def write_float(self, section, key, value):
        """ @param value: float. """
        if isinstance(value, float):
            self.__add(section, key, str(value))
        else:
            raise ValueError, '"%s", "%s" の値が float ではありません' % (section, key)
    
    def write_bool(self, section, key, value):
        """ @param value: bool. """
        if isinstance(value, bool):
            if value:
                self.__add(section, key, "1")
            else:
                self.__add(section, key, "0")
        else:
            raise ValueError, '"%s", "%s" の値が bool ではありません' % (section, key)
    
    def write_str(self, section, key, value):
        """ @param value: str. """
        if isinstance(value, str):
            self.__add(section, key, value)
        else:
            raise ValueError, '"%s", "%s" の値が str ではありません' % (section, key)

if __name__ == "__main__":
    print "こいつ自体を実効しても何も起こらないよ"
