#!/usr/bin/env python
#-*- coding:utf-8 -*-

import pygtk
pygtk.require("2.0")
import gtk
import pygst
pygst.require('0.10')
import gst
import gobject
import os
import sys
#import urllib
import inifile8
import dbus
import dbus.service
import dbus.glib
import urllib
from common import *

import y901dialog

class Y901window(dbus.service.Object):
    """
        Main Window
    """
    def __init__(self, datadir, bus_name, object_path="/org/y901x/Object1"):
        """
            @param datadir: path string
            @param bus_name: dbus name
            @param object_path: object path
        """
        # dbus初期化、dbus.glib を import しないと何故かエラー
        dbus.service.Object.__init__(self, bus_name, object_path)
        # とりあえず
        if datadir != "":
            sys.path.insert(0, datadir[:-1])
        self.w = gtk.Window()
        # attribute
        self.datadir = datadir
        self.is_playing = False
        self.p_path = ""
        self.p_filename = ""
        self.seek_at_play = False
        self.is_seek = False
        self.timer_id = -1
        self.p_position = gst.CLOCK_TIME_NONE
        self.p_duration = gst.CLOCK_TIME_NONE
        self.resize_list = 0
        self.fullscreen = False
        self.fullsize = [320, 240]
        self.full_list = False
        self.settingwin = { "list_width": 100,
                            "list_show": True,
                            "volume": 80.0,
                            "video_size": 0,
                            "repeat": 0}
        self.setting = {"exts": ".wmv|.flv|.avi|.mpg|.mpeg",
                        "position": False,
                        "init_size_on": False,
                        "init_size_val": 0,
                        "set_size": [[320,240],[640,480],[1280,720]]}
        self.listup_ext = []
        self.gesture = {"start": False,
                        "result": "",
                        "title": "",
                        "s": "",
                        "x": 0,
                        "y": 0}
        self.video = {  "width": 0,
                        "height": 0}
        # 設定ファイルの読み込み
        inipath = os.environ["HOME"] + "/.y901x"
        if os.path.exists(inipath):
            inipath += "/y901x.ini"
            if os.path.exists(inipath):
                ini = inifile8.Inifile(inipath)
                try:
                    """ [window] """
                    x = ini.read_int("window", "left", 0)
                    y = ini.read_int("window", "top", 0)
                    cx = ini.read_int("window", "width", 320)
                    cy = ini.read_int("window", "height", 320)
                    self.settingwin["list_width"]  = ini.read_int("window", "list_width", 100)
                    self.settingwin["list_show"]   = ini.read_bool("window", "list_show", True)
                    self.settingwin["volume"]      = ini.read_float("window", "volume", 80.0)
                    self.settingwin["video_size"]  = ini.read_int("window", "video_size", 0)
                    """ [setting] """
                    self.setting["position"]       = ini.read_bool("setting", "position", False)
                    self.setting["init_size_on"]   = ini.read_bool("setting", "init_size_on", False)
                    self.setting["init_size_val"]  = ini.read_int("setting", "init_size_val", 0)
                    self.setting["exts"]           = ini.read_str("setting", "exts", ".wmv|.flv|.avi|.mpg|.mpeg")
                    self.setting["set_size"][0][0] = ini.read_int("setting", "set_size1", 320)
                    self.setting["set_size"][0][1] = ini.read_int("setting", "set_size2", 240)
                    self.setting["set_size"][1][0] = ini.read_int("setting", "set_size3", 640)
                    self.setting["set_size"][1][1] = ini.read_int("setting", "set_size4", 480)
                    self.setting["set_size"][2][0] = ini.read_int("setting", "set_size5", 1280)
                    self.setting["set_size"][2][1] = ini.read_int("setting", "set_size6", 720)
                except Exception, s:
                    self.messagebox("%s" % s)
                    sys.exit()
                # 設定により移動とか
                if self.setting["init_size_on"]:
                    self.settingwin["video_size"] = self.setting["init_size_val"]
                self.w.resize(cx, cy)
                if self.setting["position"]:
                    self.w.move(x, y)
        # 抜きだし拡張子のリスト化
        self.listup_ext = self.setting["exts"].split("|")
        #
        # GtkUIManager 作成
        uimanager = gtk.UIManager()
        #
        # GtkAccelGroup を得る
        # accel_key, accel_mods, accel_flags, callback
        self.accelgroup0 = uimanager.get_accel_group()
        self.accelgroup0.connect_group(gtk.keysyms.KP_0, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_1, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_2, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_3, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_4, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_5, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_6, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_7, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_8, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.KP_9, 0, gtk.ACCEL_VISIBLE, self.on_keypad)
        self.accelgroup0.connect_group(gtk.keysyms.F12, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.F5, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.F2, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.accelgroup0.connect_group(gtk.keysyms.Escape, 0, gtk.ACCEL_VISIBLE, self.on_keydown)
        self.w.add_accel_group(self.accelgroup0)
        #
        # GtkActionGroup 作成
        self.actiongroup = gtk.ActionGroup("sasakimamenu")
        # GtkActionEntry を作成して突っ込む
        # name, stock_id, label, accelerator, tooltip, callback
        self.ac0 = [("open", gtk.STOCK_OPEN, "開く(_O)", "<Control>O", "開く", self.on_open),
                    ("play", gtk.STOCK_MEDIA_PLAY, "再生/ポーズ(_P)", "space", "再生/ポーズ", self.on_play),
                    ("stop", gtk.STOCK_MEDIA_STOP, "停止(_S)", "BackSpace", "停止", self.on_stop),
                    ("prev", gtk.STOCK_GO_UP, "手前を再生(_U)", "Up", "手前を再生", self.on_prev),
                    ("next", gtk.STOCK_GO_DOWN, "次を再生(_D)", "Down", "次を再生", self.on_next),
                    ("quit", gtk.STOCK_QUIT, "終了(_Q)", "<Control>Q", "さいなら", self.on_quit),
                    ("set", gtk.STOCK_PREFERENCES, "設定(_P)", "<Control>P", "設定", self.on_setting),
                    ("ver", gtk.STOCK_ABOUT, "情報(_B)", "<Control>B", "情報", self.on_about),
                    ("full", None, "フルスクリーン(_F)", "Z", "フルスクリーン切り替え", self.on_full),
                    ("File", None, "ファイル(_F)"),
                    ("Rep", None, "リピート(_R)"),
                    ("View", None, "表示(_V)"),
                    ("Setting", None, "設定(_P)"),
                    ("Help", None, "ヘルプ(_H)")]
                    #("Popup", None, "")]
        self.actiongroup.add_actions(self.ac0)
        # GtkRadioActionEntry の List を作成
        # name, stock_id, label, accelerator, tooltip, value
        self.ac1 = [("x00", None, "ウインドサイズ", "0", "ウインドサイズ", 0),
                    ("x10", None, "x1.0", "1", "x1.0", 1),
                    ("x20", None, "x2.0", "2", "x2.0", 2),
                    ("x30", None, "x3.0", "3", "x3.0", 3),
                    ("x05", None, "x0.5", "4", "x0.5", 4),
                    ("x15", None, "x1.5", "5", "x1.5", 5),
                    ("x25", None, "x2.5", "6", "x2.5", 6),
                    ("xs1", None, "set1", "7", "set1", 7),
                    ("xs2", None, "set2", "8", "set1", 8),
                    ("xs3", None, "set3", "9", "set1", 9) ]
        self.ac2 = [("rep_non", None, "リピート無し", None, "リピート無し", 0),
                    ("rep_one", None, "シングルリピート", "F6", "シングルリピート", 1),
                    ("rep_all", None, "すべてリピート", "F7", "すべてリピート", 2),
                    ("rep_rdm", None, "ランダム再生", "F8", "ランダム再生", 3) ]
        # GtkRadioAction を突っ込む
        # entries, value=0, on_change=None, user_data=None
        self.actiongroup.add_radio_actions(self.ac1, -1, self.on_size_change)
        self.actiongroup.add_radio_actions(self.ac2, 0, self.on_rep_change)
        #
        # GtkToggleActionEntry
        # name, stock_id, label, accelerator, tooltip, callback, is_active
        ###self.ac2 = [(
        # GtkToggleAction
        # name, label, tooltip, stock_id
        #
        uimanager.insert_action_group(self.actiongroup, 0)
        uimanager.add_ui_from_string(ui_str)
        uimanager.ensure_update()
        #
        #TODO AccelGroup: full screen と通常で切り替える
        #
        # Popup Menu を得る
        self.popup_menu = uimanager.get_widget('/popup')
        #
        # ステータスバーを作る
        self.statusbar = CStatusBar(3, self.w, ["00:00/00:00", "win", "Defaule"])
        # リストボックスを作る
        self.listbox = CListBox(self.w)
        self.listbox.set_size_request(self.settingwin["list_width"], -1)
        self.listbox.view.connect("button_release_event", self.on_Listbox_button_release)
        # ツールボックス
        self.toolbox = CToolBox()
        self.toolbox.set_volume(self.settingwin["volume"])
        self.toolbox.play.connect("clicked", self.on_play)
        self.toolbox.stop.connect("clicked", self.on_stop)
        self.toolbox.playlist.connect("clicked", self.on_list_button_clicked)
        #self.toolbox.play.connect("clicked", self.on_play)
        self.toolbox.seekbar.connect("value_changed", self.on_seek_value_changed)
        self.toolbox.seekbar.connect("button_press_event", self.on_seek_button_press_event)
        self.toolbox.seekbar.connect("button_release_event", self.on_seek_button_release_event)
        self.toolbox.volumebar.connect("value_changed", self.on_volume_value_changed)
        self.toolbox.volumebar.connect("button_press_event", self.on_volume_button_press_event)
        self.toolbox.volumebar.connect("button_release_event", self.on_volume_button_release_event)
        #
        # PlayBin 作成
        self.player = gst.element_factory_make("playbin", "player")
        self.player.set_property("volume", self.settingwin["volume"] / 100)
        bus = self.player.get_bus()
        bus.add_signal_watch()
        bus.enable_sync_message_emission()
        bus.connect("message", self.gst_on_message)
        bus.connect("sync-message::element", self.gst_on_sync_message)
        #
        # 主要コンテナの作成
        vbox_submain = gtk.HBox()
        vbox_submain.show()
        hbox_ctrl = gtk.VBox()
        hbox_ctrl.show()
        self.vbox_main = gtk.VBox()
        self.vbox_main.show()
        # Video Window を作り背景を黒にしておく
        self.video_window = gtk.DrawingArea()
        self.video_window.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(0,0,0))
        self.video_window.set_property("can_focus", True)
        # menubar
        self.menubar = uimanager.get_widget("/MenuBar")
        # パッキングしていく
        hbox_ctrl.pack_start(self.menubar, False)
        hbox_ctrl.pack_start(self.video_window)
        hbox_ctrl.pack_end(self.toolbox, False)
        hbox_ctrl.show_all()
        vbox_submain.pack_start(hbox_ctrl)
        vbox_submain.pack_end(self.listbox, False)
        self.vbox_main.pack_start(vbox_submain)
        self.vbox_main.pack_start(self.statusbar, False, False, 0)
        #
        #本体へ
        self.w.add(self.vbox_main)
        self.w.connect("delete-event", self.on_quit)
        # TitleBar
        self.icon = gtk.gdk.pixbuf_new_from_file(self.datadir + "y901x.xpm")
        self.w.set_icon(self.icon)
        self.w.set_title("Y901x")
        # D&D
        dnd_list = [("text/uri-list", 0, 0)]
        self.w.drag_dest_set(   gtk.DEST_DEFAULT_MOTION |
                                gtk.DEST_DEFAULT_HIGHLIGHT |
                                gtk.DEST_DEFAULT_DROP,
                                dnd_list, gtk.gdk.ACTION_MOVE )
        # GDK イベントの有効化を行っておく
        self.w.set_events(  gtk.gdk.BUTTON_PRESS_MASK |
                            gtk.gdk.BUTTON_RELEASE_MASK |
                            gtk.gdk.POINTER_MOTION_MASK |
                            gtk.gdk.POINTER_MOTION_HINT_MASK |
                            gtk.gdk.KEY_RELEASE_MASK |
                            gtk.gdk.STRUCTURE_MASK )
        self.w.connect("button_press_event", self.on_button_press)
        self.w.connect("button_release_event", self.on_button_release)
        self.w.connect("motion_notify_event", self.on_motion_notify)
        self.w.connect("drag_data_received", self.on_drag_data_received)
        self.w.show()
        #
        # 以下表示後でないとできな処理
        #
        # size menu のラジオメニュー位置の変更
        # この処理で on_size_change シグナルが起こるので指定サイズ変更はそちらで
        action = self.actiongroup.get_action(ac_str[self.settingwin["video_size"]])
        action.set_current_value(self.settingwin["video_size"])
        # フォーカスを移動
        self.w.set_focus(self.video_window)
        # argv の処理
        if len(sys.argv) > 1:
            if sys.argv[1].startswith('file://'):
                self.set_uri(sys.argv[1])
            else:
                self.set_uri("file://" + sys.argv[1])


########################################################################
#
#                       Functions
#
########################################################################


    def messagebox(self, text):
        dlg = gtk.MessageDialog(self.w, gtk.DIALOG_MODAL, gtk.MESSAGE_WARNING, 
                                gtk.BUTTONS_OK, text)
        dlg.set_title("確認 - Y901x")  
        r = dlg.run()  
        dlg.destroy()

    @dbus.service.method("org.y901x.Interface1")
    def set_uri(self, uri):
        # 一旦 NULL にしないと失敗する
        # 未対応ファイルは gst_on_message で
        self.player.set_state(gst.STATE_NULL)
        self.is_playing = False
        self.player.set_property("uri", uri)
        self.player.set_state(gst.STATE_PLAYING)

    def close(self):
        self.player.set_state(gst.STATE_NULL)
        # ドットディレクトリが存在しなければ作る
        inipath = os.environ["HOME"] + "/.y901x"
        if not os.path.exists(inipath):
            os.mkdir(inipath)
        # y901x.ini があれば読み込んでおく
        inipath += "/y901x.ini"
        ini = inifile8.Inifile(inipath)
        try:
            """ [window] """
            x, y = self.w.get_position()
            ini.write_int("window", "left", x)
            ini.write_int("window", "top", y)
            if self.fullscreen:
                ini.write_int("window", "width", self.fullsize[0])
                ini.write_int("window", "height", self.fullsize[1])
            else:
                cx, cy = self.w.get_size()
                ini.write_int("window", "width", cx)
                ini.write_int("window", "height", cy)
            ini.write_int("window", "list_width", self.listbox.get_property("width_request"))
            ini.write_bool("window", "list_show", self.listbox.get_property("visible"))
            ini.write_float("window", "volume", self.player.get_property("volume") * 100)
            ini.write_int("window", "video_size", self.settingwin["video_size"])
            """ [setting] """
            ini.write_bool("setting", "position", self.setting["position"])
            ini.write_bool("setting", "init_size_on", self.setting["init_size_on"])
            ini.write_int("setting", "init_size_val", self.setting["init_size_val"])
            ini.write_str("setting", "exts", self.setting["exts"])
            ini.write_int("setting", "set_size1", self.setting["set_size"][0][0])
            ini.write_int("setting", "set_size2", self.setting["set_size"][0][1])
            ini.write_int("setting", "set_size3", self.setting["set_size"][1][0])
            ini.write_int("setting", "set_size4", self.setting["set_size"][1][1])
            ini.write_int("setting", "set_size5", self.setting["set_size"][2][0])
            ini.write_int("setting", "set_size6", self.setting["set_size"][2][1])
            # 保存
            ini.save()
        except Exception, s:
            self.messagebox("%s" % s)
        gtk.main_quit()

    def set_play_position(self, location):
        event = gst.event_new_seek( 1.0, gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_ACCURATE,
                                    gst.SEEK_TYPE_SET, location, gst.SEEK_TYPE_NONE, 0)
        self.player.send_event(event)

    def clear(self):
        self.p_position = 0
        self.is_playing = False;
        if self.timer_id != -1:
            gobject.source_remove(self.timer_id)
            self.timer_id = -1
        if self.p_duration != gst.CLOCK_TIME_NONE:
            self.player.set_state(gst.STATE_NULL)
            self.toolbox.seekadj.set_value(0)
            allsec = self.p_duration / 1000000000
            secv = "00:00/%02d:%02d" % ((allsec / 60), (allsec % 60))
            self.statusbar.label[0].set_text(secv)
            #self.w.set_title("Y901x")
            # ボタンの変更
            self.toolbox.change_img(0)
        # Wバッファリング抑制解除
        self.video_window.set_flags(gtk.DOUBLE_BUFFERED)

    def change_video_size(self):
        val = self.settingwin["video_size"]
        if val == 0:
            return
        elif val < 7:
            if self.player.get_property("uri") == None:
                return
        # 計算
        menu_v = self.menubar.get_allocation().height
        ctrl_v = self.toolbox.get_allocation().height
        stat_v = self.statusbar.get_allocation().height
        x = self.video["width"]
        y = self.video["height"]
        # 設定により振り分け
        if val == 2: # x2.0
            x *= 2
            y *= 2
        elif val == 3: # x3.0
            x *= 3
            y *= 3
        elif val == 4: # x0.5
            x /= 2
            y /= 2
        elif val == 5: # x1.5
            x = x * 3 / 2
            y = y * 3 / 2
        elif val == 6: # x2.5
            x = x * 5 / 2
            y = y * 5 / 2
        elif val == 7: # set1
            x = self.setting["set_size"][0][0]
            y = self.setting["set_size"][0][1]
        elif val == 8: # set2
            x = self.setting["set_size"][1][0]
            y = self.setting["set_size"][1][1]
        elif val == 9: # set3
            x = self.setting["set_size"][2][0]
            y = self.setting["set_size"][2][1]
        if x ==0 or y == 0:
            return
        # メニューやコントローラのサイズを加算
        # リストは表示されている場合のみ、15 はセパレータの合計サイズ
        if self.listbox.get_property("visible"):
            list_h = self.listbox.get_property("width_request") + 15
            x += list_h
        y += menu_v + ctrl_v + stat_v
        # 現在のサイズと違っていたらリサイズさせる、整数にしないと警告になる
        cx, cy = self.w.get_size()
        if cx != x or cy != y:
            self.w.resize(x, y)

    def listup_change(self):
        uri = self.player.get_property("uri")
        t = urllib.unquote(uri)
        p_path, self.p_filename = os.path.split(t)
        # パスが変わっていたら or リストにファイル名無しなら読み直し
        if self.p_path != p_path[7:] or self.listbox.is_text(self.p_filename) == False:
            self.p_path = p_path[7:]
            self.listbox.liststore.clear()
            files = os.listdir(self.p_path)
            files.sort()
            for f in files:
                # isfile and 拡張子一致のみ追加
                if os.path.isfile(self.p_path +"/" + f):
                    ext = os.path.splitext(f)[1].lower()
                    # 拡張子が登録されていなくても一致ファイルだけは追加
                    if ext in self.listup_ext or f == self.p_filename:
                        it = self.listbox.liststore.append([f])
                        # セレクタの変更
                        if f == self.p_filename:
                            self.listbox.select_text(self.p_filename)
        else:
            # 同じディレクトリにあるファイルをドロップ
            # なんて場合があるので一応変更しておく
            self.listbox.select_text(self.p_filename)
        self.w.set_title(self.p_filename + " - Y901x")

    def rename(self):
        if self.p_filename == "":
            return
        name = self.p_filename
        label = gtk.Label()
        label.set_text(self.p_filename)
        entry = gtk.Entry()
        entry.set_text(self.p_filename)
        d = gtk.Dialog( "リネーム",
                        self.w,
                        gtk.DIALOG_MODAL,# | gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                        gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
        d.vbox.pack_start(label, False)
        d.vbox.pack_start(entry, False)
        d.show_all()
        # GtkEntry にて Enter キーで OK ボタンを押す処理
        def dlg_ok(self):
            d.response(gtk.RESPONSE_ACCEPT)
        entry.connect("activate", dlg_ok)
        # 再生中ならポーズ、リピート中だと次に行ってしまう
        is_play = False
        if self.player.get_state(1)[1] == gst.STATE_PLAYING:
            is_play = True
            self.player.set_state(gst.STATE_PAUSED)
        # 開始
        if d.run() == gtk.RESPONSE_ACCEPT:
            text = entry.get_text()
            if text == self.p_filename:
                self.messagebox("変更されていません")
            else:
                if  text in os.listdir(self.p_path):
                    self.messagebox("同一ファイル名が見つかりました")
                else:
                    newname = self.p_path + "/" + text
                    self.clear()
                    while gtk.events_pending():
                        gtk.main_iteration()
                    os.rename(self.p_path + "/" + self.p_filename, newname)
                    while gtk.events_pending():
                        gtk.main_iteration()
                    self.set_uri("file://" + newname)
        d.destroy()
        if is_play:
            self.player.set_state(gst.STATE_PLAYING)

    def change_fullscreen(self, b):
        if self.p_filename == "":
            return
        if b:
            self.fullscreen = False
            self.w.window.unfullscreen()
            self.w.unfullscreen()
            self.toolbox.show()
            self.menubar.show()
            self.statusbar.show()
            if self.full_list:
                self.listbox.show()
            self.w.resize(self.fullsize[0], self.fullsize[1])
        else:
            self.fullscreen = True
            cx, cy = self.w.get_size()
            self.fullsize[0] = cx
            self.fullsize[1] = cy
            self.full_list = self.listbox.get_property("visible")
            if self.full_list:
                self.listbox.hide()
            self.menubar.hide()
            self.statusbar.hide()
            self.toolbox.hide()
            self.w.fullscreen()
            self.w.window.fullscreen()


########################################################################
#
#                       Menu Signal Handlers
#
########################################################################


    def on_open(self, widget, event=None):
        dlg = gtk.FileChooserDialog("ファイルを開く", self.w,
                                    gtk.FILE_CHOOSER_ACTION_OPEN,
                                    (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, 
                                    gtk.STOCK_OPEN, gtk.RESPONSE_OK))
        r = dlg.run()
        if r == gtk.RESPONSE_OK:
            self.set_uri("file://" + dlg.get_filename())
        dlg.destroy()

    def on_play(self, widget, event=None):
        if self.player.get_state(1)[1] == gst.STATE_PLAYING:
            self.player.set_state(gst.STATE_PAUSED)
        else:
            if self.player.get_property("uri") != None:
                if self.player.set_state(gst.STATE_PLAYING) == gst.STATE_CHANGE_FAILURE:
                    self.messaagebox("再生できませんでした")

    def on_stop(self, widget, event=None):
        self.clear()

    def on_prev(self, widget, event=None):
        pass

    def on_next(self, widget, event=None):
        pass

    def on_rep_change(self, action, current):
        pass

    def on_quit(self, widget, event=None):
        self.close()
        return True

    def on_full(self, widget, event=None):
        self.change_fullscreen(self.fullscreen)

    def on_size_change(self, action, current):
        # ステータスバーにサイズの表示
        num = action.get_current_value()
        self.settingwin["video_size"] =  num
        self.statusbar.label[1].set_text(size_str[num])
        self.change_video_size()

    def on_setting(self, widget, event=None):
        try:
            dlg = y901dialog.Y901dialog(self.w, self.setting, self.datadir) 
        except:
            self.messagebox(self.datadir + "ダイアログファイルが見つかりません")
            return
        r = dlg.run()
        if r == gtk.RESPONSE_OK:
            self.setting["position"] = dlg["checkbutton_pos"].get_active()
            self.setting["init_size_on"] = dlg["checkbutton_init_size_on"].get_active()
            self.setting["init_size_val"] = dlg["combobox_size"].get_active()
            exts = dlg["entry_ext"].get_text().lower()
            if exts != self.setting["exts"]:
                self.setting["exts"] = exts
                self.listup_ext = self.setting["exts"].split("|")
                self.listup_change()
            self.setting["set_size"][0][0] = int(dlg["spinbutton1"].get_value())
            self.setting["set_size"][0][1] = int(dlg["spinbutton2"].get_value())
            self.setting["set_size"][1][0] = int(dlg["spinbutton3"].get_value())
            self.setting["set_size"][1][1] = int(dlg["spinbutton4"].get_value())
            self.setting["set_size"][2][0] = int(dlg["spinbutton5"].get_value())
            self.setting["set_size"][2][1] = int(dlg["spinbutton6"].get_value())
        dlg.destroy()
        self.w.set_focus(self.video_window)

    def on_about(self, widget, event=None):
        url = "http://palepoli.skr.jp/"
        w = gtk.AboutDialog()
        w.set_transient_for(self.w)
        w.set_program_name("Y901x")
        w.set_version("0.1.0")
        w.set_copyright("Copyright(C) 2009 sasakima-nao")
        w.set_license("BSD の予定、今は無い")
        w.set_logo(self.icon)
        # コメントは得に入れたものがないので GTK のバージョンでも
        v = "GTK+ Version %d.%d.%d\n" % gtk.ver
        v += "PyGtk Version %d.%d.%d" % gtk.pygtk_version
        w.set_comments(v)
        # set_url_fook は website より先に書かないと認識しない
        def about_url(about, link, data): os.system("xdg-open %s" % link)
        gtk.about_dialog_set_url_hook(about_url, url)
        w.set_website(url)
        w.set_website_label(url)
        w.run()
        w.destroy()

########################################################################
#
#                   MainWindow Signal Handlers
#
########################################################################


    def on_list_button_clicked(self, widget, event=None):
        if self.fullscreen:
            return
        width = self.listbox.get_property("width_request")
        cx, cy = self.w.get_size()
        if self.listbox.get_property("visible"):
            self.listbox.hide()
            self.w.resize(cx - width, cy)
        else:
            self.listbox.show()
            self.w.resize(cx + width, cy)
    
    def on_button_press(self, widget, event):
        # マウスジェスチャ開始
        if event.button == 3:
            self.gesture["start"] = True
            self.gesture["x"] = event.x_root
            self.gesture["y"] = event.y_root
            self.gesture["result"] = ""
            self.gesture["s"] = ""
            self.gesture["title"] = ""
        elif event.type == gtk.gdk._2BUTTON_PRESS:
            self.change_fullscreen(self.fullscreen)
    
    def on_motion_notify(self, widget, event):
        # マウスジェスチャ中
        if self.gesture["start"]:
            x = abs(event.x_root - self.gesture["x"])
            y = abs(event.y_root - self.gesture["y"])
            s = ""
            if (x > 25) or (y > 25):
                if x > y:
                    if (self.gesture["x"] - event.x_root) > 0:
                        s = "←"
                    else:
                        s = "→"
                else:
                    if (self.gesture["y"] - event.y_root) > 0:
                        s = "↑"
                    else:
                        s = "↓"
                if self.gesture["s"] != s:
                    self.gesture["s"] = s
                    self.gesture["result"] += s
                self.gesture["x"] = event.x_root
                self.gesture["y"] = event.y_root
                if self.gesture["title"] == "":
                    self.gesture["title"] = self.w.get_title()
                self.w.set_title(self.gesture["result"])
    
    def on_button_release(self, widget, event):
        # まず gesture check、if event.button == 3: だと右メニュー時に不具合
        if self.gesture["start"]:
            self.gesture["start"] = False
            if self.gesture["result"] == "":
                # 右クリでかつジェスチャ無しなら PopupMenu
                self.popup_menu.popup(None, None, None, event.button, event.time)
            else:
                # 忘れずに
                if self.gesture["title"] != "":
                    self.w.set_title(self.gesture["title"])
                # 処理
                if self.gesture["result"] == "→":
                    self.on_next(widget,0)
                elif self.gesture["result"] == "←":
                    self.on_prev(widget,0)
                elif self.gesture["result"] == "↓→":
                    self.close()

    def on_drag_data_received(self, widget, context, x, y, selection_data, info, time):
        # D&D
        if selection_data and selection_data.data:
            value = selection_data.data.split("\n")
            self.set_uri(value[0][:-1])


########################################################################
#
#                       About Signal Handlers
#
########################################################################


    def on_timer(self):
        try:
            self.p_position = self.player.query_position(gst.FORMAT_TIME)[0]
        except:
            self.p_position = gst.CLOCK_TIME_NONE
        if self.p_position != gst.CLOCK_TIME_NONE:
            self.toolbox.seekadj.set_value(self.p_position)
            sec = self.p_position / 1000000000
            allsec = self.p_duration / 1000000000
            secv = "%02d:%02d/%02d:%02d" % ((sec / 60), (sec % 60), (allsec / 60), (allsec % 60))
            self.statusbar.label[0].set_text(secv)
        # Trueを返すと時間ごとに繰り返し実行される
        return self.is_playing

    def on_seek_value_changed(self, widget, event=None):
        if self.timer_id == -1:
            return
        if self.is_seek:
            #location = widget.get_value()
            location = widget.get_value()
            self.set_play_position(location)
            sec = location / 1000000000
            allsec = self.p_duration / 1000000000
            secv = "%02d:%02d/%02d:%02d" % ((sec / 60), (sec % 60), (allsec / 60), (allsec % 60))
            self.statusbar.label[0].set_text(secv)

    def on_seek_button_press_event(self, widget, event):
        if self.timer_id == -1:
            return
        if self.player.get_state(1)[1] == gst.STATE_PLAYING:
            self.player.set_state(gst.STATE_PAUSED)
            self.seek_at_play = True
        gobject.source_remove(self.timer_id)
        self.is_seek = True
        event.button = 2

    def on_seek_button_release_event(self, widget, event):
        self.w.set_focus(self.video_window)
        if self.timer_id == -1:
            return
        self.is_seek = False
        if self.seek_at_play:
            self.seek_at_play = False
            self.player.set_state(gst.STATE_PLAYING)
        self.timer_id = gobject.timeout_add(200, self.on_timer)
        event.button = 2

    def on_volume_value_changed(self, widget, event=None):
        val = widget.get_value() / 100
        self.player.set_property("volume", val)

    def on_volume_button_press_event(self, widget, event):
        event.button = 2

    def on_volume_button_release_event(self, widget, event):
        self.w.set_focus(self.video_window)
        event.button = 2

    def on_Listbox_button_release(self, widget, event=None):
        model, it = self.listbox.selection.get_selected()
        if it == None:
            return
        s = model.get_value(it, 0)
        if s != self.p_filename:
            new_uri = "file://" + self.p_path + "/" + s
            self.set_uri(new_uri)
        self.w.set_focus(self.video_window)

    def on_keypad(self, accelGroup, window, keyval, modifier):
        # フルスクリーンなら解除
        if self.fullscreen:
            self.change_fullscreen(self.fullscreen)
        if keyval == 65456:
            self.settingwin["video_size"] = 0
            self.statusbar.label[1].set_text(size_str[0])
        elif keyval == 65457:
            self.settingwin["video_size"] = 1
            self.statusbar.label[1].set_text(size_str[1])
        elif keyval == 65458:
            self.settingwin["video_size"] = 2
            self.statusbar.label[1].set_text(size_str[2])
        elif keyval == 65459:
            self.settingwin["video_size"] = 3
            self.statusbar.label[1].set_text(size_str[3])
        elif keyval == 65460:
            self.settingwin["video_size"] = 4
            self.statusbar.label[1].set_text(size_str[4])
        elif keyval == 65461:
            self.settingwin["video_size"] = 5
            self.statusbar.label[1].set_text(size_str[5])
        elif keyval == 65462:
            self.settingwin["video_size"] = 6
            self.statusbar.label[1].set_text(size_str[6])
        elif keyval == 65463:
            self.settingwin["video_size"] = 7
            self.statusbar.label[1].set_text(size_str[7])
        elif keyval == 65464:
            self.settingwin["video_size"] = 8
            self.statusbar.label[1].set_text(size_str[8])
        elif keyval == 65465:
            self.settingwin["video_size"] = 9
            self.statusbar.label[1].set_text(size_str[9])
        self.change_video_size()

    def on_keydown(self, accelGroup, window, keyval, modifier):
        # メニューに無いキーボード操作まとめて
        if keyval == gtk.keysyms.F12:
            self.on_list_button_clicked(window)
        elif keyval == gtk.keysyms.F5:
            self.p_path = ""
            self.listup_change()
        elif keyval == gtk.keysyms.F2:
            self.rename()
        elif keyval == gtk.keysyms.Escape:
            self.close()


########################################################################
#
#                       GStreamer Event Handlers
#
########################################################################
    
        
    def gst_on_message(self, bus, message):
        pass
    def gst_on_message(self, bus, message):
        t = message.type
        if t == gst.MESSAGE_EOS:
            self.clear()
            # TODO リピートの処理
            """if self["togglebutton_rep_all"].get_active():
                self.gtk_on_menuitem_down_activate(self)
            elif self["togglebutton_rep_one"].get_active():
                self.set_play_position(0)
                self.player.set_state(gst.STATE_PLAYING)"""
        elif t == gst.MESSAGE_ERROR:
            self.clear()
            err, debug = message.parse_error()
            self.messagebox(str(err) + "\n" + str(debug))  
            self.toolbox.change_img(0)
            self.player.set_property("uri", None)
        elif t == gst.MESSAGE_STATE_CHANGED:
            if gst.STATE_PLAYING == message.parse_state_changed()[1]:
                self.toolbox.change_img(0)
                # 初回のみ以下を実効
                if not self.is_playing:
                    try:
                        self.p_duration = self.player.query_duration(gst.FORMAT_TIME)[0]
                    except:
                        self.p_duration = gst.CLOCK_TIME_NONE
                    if self.p_duration != gst.CLOCK_TIME_NONE:
                        self.toolbox.seekadj.upper = self.p_duration
                        self.is_playing = True
                        # Wバッファリング抑制、チラツキ防止はココで行う
                        self.video_window.unset_flags(gtk.DOUBLE_BUFFERED)
                        # Timer on
                        self.timer_id = gobject.timeout_add(200, self.on_timer)
                        # FileName 抜きだし
                        self.listup_change()
                        # stream-info リストから元サイズ取得
                        info_arr = self.player.get_property("stream-info-value-array")
                        for info in info_arr:
                            pad = info.get_property("object")
                            caps = pad.get_negotiated_caps()
                            if caps != None and "video" in caps.to_string():
                                self.video["width"] = caps[0]["width"]
                                self.video["height"] = caps[0]["height"]
                                self.change_video_size()
            else:
                self.toolbox.change_img(1)
                #if self["image_play"].get_property("file") == self.datadir + "img/pausebutton.xpm":
                #    self["image_play"].set_from_file(self.datadir + "img/playbutton.xpm");

    def gst_on_sync_message(self, bus, message):
        if message.structure is None:
            return
        message_name = message.structure.get_name()
        if message_name == "prepare-xwindow-id":
            imagesink = message.src
            imagesink.set_property("force-aspect-ratio", True)
            imagesink.set_xwindow_id(self.video_window.window.xid)


########################################################################
#
#                               main
#
########################################################################


def main():
    try:
        # セッションバスを開く
        session_bus = dbus.SessionBus()
        # サービスからオブジェクトを取得、無ければ例外
        remote_object = session_bus.get_object("org.y901x.Service1", "/org/y901x/Object1")
        # notify_startup_complete 用にやっておく
        while gtk.events_pending():
            gtk.main_iteration()

        if len(sys.argv) > 1:
            # インタフェースを通して関数を実効
            if sys.argv[1].startswith('file://'):
                uri = sys.argv[1]
            else:
                uri = "file://" + sys.argv[1]
            try:
                remote_object.set_uri(uri, dbus_interface = "org.y901x.Interface1")
            except:
                print("Y901x: Error remote_object.set_uri")
        # この指定をしないとタスクバーに現れる
        try:
            gtk.gdk.notify_startup_complete()
        except:
            print("Y901x: Error notify_startup_complete")
        return
    except:
        bus_name = dbus.service.BusName("org.y901x.Service1", bus=session_bus)
        w = Y901window("", bus_name)
        gtk.main()

if __name__ == "__main__":
    main()
